%---------------------------------------------
%Analyzator HRV.
%Skript provadi analyzu HRV v oblasti casove (RMSSD, SDSD, pNN50, TF) a v
%oblasti frekvencni, kde jsou pocitany vykony v jednotlivych HRV pasmech -
%TP, P_HF, P_LF, P_VLF, P_HF/P_LF

%Nastavovat lze zacatek a konec analyzy v ms, vzorkovaci kmitocet, dale lze
%zapnout/vypnout kontrolu celistvosti signalu pomoci dvou metod, lze
%povolit nebo zakazat odstranovani artefaktu v signalu a ovladat vyznaceni
%HRV pasem do spektra HRV

%Skript ocekava na vstupu .csv soubor, ktery ma nasledujici format:
%1. sloupec - EKG signal (rozsah 0 a 4095)
%2. sloupec - ACCEL osa x (rozsah -2048 a 2047)
%3. sloupec - ACCEL osa y (rozsah -2048 a 2047)
%4. sloupec - ACCEL osa z (rozsah -2048 a 2047)
%5. sloupec - kontrolni pilovy signal (rozsah 0 a 255)
%---------------------------------------------
%Ke skriptu je prilozen soubor "EKG_zkusebni.csv", ve kterem je dvouminutovy
%zaznam signalu EKG (muz 24 let). Zaznam obsahuje jeden artefakt (extrasystolu). Na
%tomto zaznamu je mozne skript vyzkouset.
%---------------------------------------------
clc
clear all
%--------------------------------------------- ovladani (1 - zapnuto, 0 - vypnuto)
zacatek = 100; % ms
konec = 120000; % ms

Fs = 1000; % vzorkovaci frekvence

Pila_rychla_ano = 1;    % rychla kontrola signalu pomoci xcorr
Pila_bod_po_bodu_ano = 0; % uplna kontrola signalu bod po bodu s vypisem poctu chyb

Odstran_artefakty = 1; % odstraneni artefaktu 
Hranice_ano = 1; % zobrazeni hranic jednotlivych HRV pasem ve spektru HRV
%---------------------------------------------


[fileName, pathName] = uigetfile('*.csv'); 
Mat = fullfile(pathName,fileName);
M = importdata(Mat); 



[radky, sloupce] = size(M);

i = 1;

while(i < radky) % kontrola rozsahu dat
    
    i = i+1;
    if(M(i,1) > 4095)
        M(i,1) = M(i-1,1);
    end
    
    if(M(i,2) > 2047 || M(i,2) < -2048)
        M(i,2) = M(i-1,2);
    end
    
    if(M(i,3) > 2047 || M(i,3) < -2048)
        M(i,3) = M(i-1,3);
    end
    
   if(M(i,4) > 2047 || M(i,4) < -2048)
        M(i,4) = M(i-1,4);
    end
    
    
end


Mekg1 = M(:,1);
Mpila = M(:,5);

%------------------------------------- interpolace na 1000 Hz
if(Fs < 1000)
    fa = [1:(1000/Fs):length(Mekg1)*(1000/Fs)];
    xx2a = [fa(1):1:fa(end)];
    Mekg = interp1(fa,Mekg1,xx2a,'linear');
%------------------------------------- decimace na 1000 Hz
elseif(Fs > 1000)
    Mekg = decimate(Mekg1,(Fs/1000));
%-------------------------------------
else
    Mekg = Mekg1;
end
%-------------------------------------




if(Pila_bod_po_bodu_ano == 1) % kontrola signalu bod po bodu
    pocet_chyb = 0;
    i = 1;
    while (i < length(Mpila(zacatek:konec)))
        if (((Mpila(zacatek + (i+1)) - Mpila(zacatek + i)) ~= 1)) 
            if ((Mpila(zacatek + (i+1)) - Mpila(zacatek + i)) ~= -255)
                pocet_chyb = pocet_chyb + 1;  
            end
        end
        i = i + 1;
    end

pocet_chyb  
chybovost = pocet_chyb / length(Mpila(zacatek:konec))
end


i = 1; % vytvoreni idealniho piloveho signalu
Kpila(1) = 0;
while(i < (256 * 1))
    Kpila(i + 1) = Kpila(i) + 1; 
    if (Kpila(i) == 255)
        Kpila(i + 1) = 0;
    end
    i = i + 1;
end

% Pan-Tompkins - detekce QRS + rychla kontrola signalu pomoci xcorr
[qrs_amp_raw, qrs_i_raw, delay]=pan_tompkin_uprava(Mekg(zacatek:konec),1000,1, Mpila(zacatek:konec), Kpila, Pila_rychla_ano);


%----------------------------------------- vypocet RR intervalu
i = 1;
   while(i < length(qrs_i_raw))
    t_RR(i) = (qrs_i_raw(i + 1) - qrs_i_raw(i)); % casy mezi RR intervaly
    i = i + 1;
   end
prumerny_RR = mean(t_RR);
%----------------------------------------- odstrann artefaktu
if(Odstran_artefakty == 1)
    i = 1;
    e = 0;
    %t_RR
    while(i < (length(t_RR))-2)
         if(abs(1 - (t_RR(i)/t_RR(i+1))) >= 0.12)  
        
            t_RR(i+1) = (((t_RR(i) - t_RR(i+3)) / 4) * 3) + t_RR(i+3);
            e = e + 1;
         end    
        i = i + 1;
    
    end
    Procento_artefaktu = (e / (length(t_RR) * 2)) * 100
    t_RRn = t_RR / 1000;

end


  %-------------------------------- vypocet HRV parametru v casove oblasti
   %-------------------------------- RMSSD   
   i = 1;
   while(i < length(t_RR))
    RRdiff(i) = (t_RR(i) - t_RR(i + 1));
    RMSSD2(i) = RRdiff(i)^2;
    i = i + 1;
   end
   
   RMSSD = sqrt(sum(RMSSD2) / (length(t_RR)-1));  
   %-------------------------------- IBI   
   mean_IBI = mean(t_RR); 
   
   %-------------------------------- SDRR
   i = 1;
   while(i < (length(t_RR) + 1))
    SDRR1(i) = (t_RR(i) - mean_IBI)^2;
    i = i + 1;
   end
   
   SDRR = sqrt(sum(SDRR1) / (length(SDRR1)-1)); 
   %-------------------------------- SDSD  
   RRdiff_mean = mean(RRdiff);
   
   i = 1;   
   while(i < (length(RRdiff) + 1))
    SDSD1(i) = (RRdiff(i) - RRdiff_mean)^2;
    i = i + 1;
   end
   
   SDSD = sqrt(sum(SDSD1) / (length(SDSD1)-1)); 
   
   %-------------------------------- pNN50
   i = 1;
   NN50 = 0;
   while(i < length(t_RR))
     if (abs(t_RR(i+1)-t_RR(i)) > 50)
         NN50 = NN50 + 1;
     end
    i = i + 1;
   end
   
   pNN50 = (NN50/(length(t_RR) - 1)) * 100;   

   TF = (((length(qrs_i_raw) * 1000)/(konec - zacatek)) * 60);
   
   if(Odstran_artefakty == 1)
       SDNN = SDRR
   else
       SDRR
   end
   
   RMSSD
   SDSD
   pNN50
   mean_IBI;
   TF         

%----------------------------------------- Histogram
i = 1;
while(i < (length(t_RR) + 1))
    fin2(i) = 1000 / t_RR(i); 
    i = i + 1;
end

figure
hodnotyY = ones(1, length(t_RR));
t_RR_max = max(t_RR);
t_RR_min = min(t_RR);
hist(t_RR, (t_RR_max - t_RR_min) / 8)
xlabel('NN intervaly [ms]');
ylabel('n [-]');
title('Histogram delek NN intervalu');

%---------------------------------------- FFT - RR interval proklad 
hodnotyX2 = qrs_i_raw(2:end);

vzorkovacka = 100;
xx2 = [hodnotyX2(1):(1000/vzorkovacka):hodnotyX2(end)];
yy3 = interp1(hodnotyX2,t_RR,xx2,'cubic');
yy2 = interp1(hodnotyX2,t_RR,xx2,'linear');

figure
plot(hodnotyX2,t_RR,'o')
hold on
%plot(xx2,yy2,'r')
plot(xx2,yy3,'r')
xlabel('t [ms]');
ylabel('NN intervaly [ms]');
title('Zavislost NN intervalu na case vln R.');

p = polyfit(hodnotyX2, t_RR, 5);


y1 = polyval(p,xx2);
plot(xx2,y1,'k') % detrendovaci funkce

yyy = yy2 - y1; % detrendovany tachogram RR intervalu
%{
figure 
plot(xx2,yyy,'r')
xlabel('t [ms]');
ylabel('NN intervaly [ms]');
title('Detrendovan zvislost NN interval na ase vln R');
%}


fs = vzorkovacka;

L = length(xx2);
%---------------------------------------- spektrum detrendovaneho signalu 
xdft = fft(yyy/1000);
xdft = xdft(1:int32(L)/2+1);
psdx = (1/(fs*L)) * abs(xdft).^2;
psdx(2:end-1) = 2*psdx(2:end-1);
psdx(1) = 0;
%----------------------------------------
f = fs*(0:(L/2))/L;

i = 1;
while(f(i) < 0.5)    
    i = i + 1;
end

f = f(1:i);
psdx = psdx(1:i);

vzorkovacka = 1000;
f2 = [f(1):(1/vzorkovacka):f(end)];
psdx2 = interp1(f,psdx,f2,'cubic');
psdx3 = interp1(f,psdx,f2,'linear');

figure
plot(f2,psdx2,'k') 
xlim([0 0.5])
xlabel('f [Hz]');
ylabel('PSD [s^2/Hz]');
title('HRV spektrum');
if(Hranice_ano == 1)
    hold on
    hraniceX = [0.0033 0.04 0.15 0.4];
    hraniceY = [max(psdx) max(psdx) max(psdx) max(psdx)];
    stem(hraniceX, hraniceY,'r')
end
%----------------------------- TP
i = 1;
while(f2(i) < 0.4)
TP_s(i) = ((f2(i+1)-f2(i))*((psdx2(i)+psdx2(i+1))/2));
    i = i + 1;
end

TP_s = sum(TP_s);
TP_ms = 1000000*TP_s
%----------------------------- HF
i = 1;
while(f2(i) < 0.15)    
    i = i + 1;
end

while(f2(i) < 0.4)
P_HF(i) = ((f2(i+1)-f2(i))*((psdx2(i)+psdx2(i+1))/2));
    i = i + 1;
end

P_HF = sum(P_HF);
P_HF_ms = 1000000*P_HF
%----------------------------- LF
i = 1;
while(f2(i) < 0.04)    
    i = i + 1;
end

while(f2(i) < 0.15)
P_LF(i) = ((f2(i+1)-f2(i))*((psdx2(i)+psdx2(i+1))/2));
    i = i + 1;
end

P_LF = sum(P_LF);
P_LF_ms = 1000000*P_LF
%----------------------------- VLF
i = 1;
while(f2(i) < 0.0033)    
    i = i + 1;
end

while(f2(i) < 0.04)
P_VLF(i) = ((f2(i+1)-f2(i))*((psdx2(i)+psdx2(i+1))/2));
    i = i + 1;
end

P_VLF = sum(P_VLF);
P_VLF_ms = 1000000*P_VLF

PLF_PHF = P_LF_ms/P_HF_ms